<?php
namespace Ep\Http;

use Ep\Http\Middleware\Cache;
use Ep\Http\Validator\ValidatorChainFactory;
use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\ClientInterface as HttpClientInterface;
use GuzzleHttp\Handler\CurlHandler;
use GuzzleHttp\HandlerStack;
use Interop\Container\ContainerInterface;
use Psr\Cache\CacheItemPoolInterface;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu http klijenta definiranog u konfiguraciji.
 *
 * @package Ep\Http
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AbstractHttpClientFactory implements FactoryInterface
{
    const CONFIG_KEY = 'http_clients';

    /** @var array */
    private $config;

    /**
     * Stvara novog http klijenta.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return HttpClientInterface
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $options ?? $this->getConfig($container)[$requestedName] ?? [];

        $handler = $options['handler'] ?? new CurlHandler();
        $stack = HandlerStack::create($handler);

        if (isset($config['validators'])) {
            $factory = new ValidatorChainFactory();
            $validator = $factory($container, '', [
                'validators' => $config['validators'],
            ]);

            $stack->push($validator);
            unset($config['validators']);
        }

        if (isset($config['cache']) && !empty($config['cache'])) {
            $cache = $config['cache'];

            if (!$cache instanceof CacheItemPoolInterface) {
                /** @var CacheItemPoolInterface $cache */
                $cache = $container->get($config['cache']);
            }

            $stack->push(new Cache($cache, $config['cache_lifetime'] ?? 86400));
            unset($config['cache']);
        }

        $config['handler'] = $stack;

        if (!isset($config['http_errors'])) {
            $config['http_errors'] = false;
        }

        return new HttpClient($config);
    }

    /**
     * Da li imamo konfiguraciju za dato ime?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        if ('config' === $requestedName) {
            return false;
        }

        $config = $this->getConfig($container);

        if (empty($config)) {
            return false;
        }

        return isset($config[$requestedName]);
    }

    /**
     * Izvlači konfiguracije http klijenata iz datog containera.
     *
     * @param ContainerInterface $container
     *
     * @return array
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    protected function getConfig(ContainerInterface $container)
    {
        if (null === $this->config) {
            $this->config = $container->has('config')
                ? $container->get('config')[self::CONFIG_KEY] ?? []
                : [];
        }

        return $this->config;
    }
}