<?php
namespace EpTest\Cache\Adapter;

use Ep\Cache\AbstractCacheFactory;
use Ep\Cache\Adapter\PluginManager;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Cache\Adapter;
use Zend\ServiceManager\ServiceManager;

/**
 * Class AdapterPluginManagerTest
 *
 * @package EpTest\Cache\Storage
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class AdapterPluginManagerTest extends TestCase
{
    /** @var PluginManager */
    private $adapters;

    protected function setUp()
    {
        parent::setUp();

        $container      = new ServiceManager([
            'abstract_factories' => [
                AbstractCacheFactory::class,
            ],
        ]);

        $this->adapters = new PluginManager($container);

        $container->setService('config', [
            'caches' => [
                'proxied' => [
                    'adapter' => 'null',
                ],
            ],
        ]);
    }

    public function testCanCreateApcuAdapter()
    {
        $this->assertTrue($this->adapters->has('apcu'));

        $adapter = $this->adapters->get('apcu');

        $this->assertInstanceOf(Adapter\ApcuAdapter::class, $adapter);
        $this->assertNotSame($adapter, $this->adapters->get('apcu'));
    }

    public function testCanCreateArrayAdapter()
    {
        $this->assertTrue($this->adapters->has('array'));

        $adapter = $this->adapters->get('array');

        $this->assertInstanceOf(Adapter\ArrayAdapter::class, $adapter);
        $this->assertNotSame($adapter, $this->adapters->get('array'));
    }

    public function testCanCreateChainAdapter()
    {
        $this->assertTrue($this->adapters->has('chain'));

        /** @var Adapter\ChainAdapter $adapter */
        $adapter = $this->adapters->get('chain', [
            'adapters' => [
                'apcu',
                'array' => [
                    'default_lifetime' => 300,
                ],
            ],
        ]);

        $this->assertInstanceOf(Adapter\ChainAdapter::class, $adapter);
    }

    public function testCanCreateFilesystemAdapter()
    {
        $this->assertTrue($this->adapters->has('filesystem'));

        $adapter = $this->adapters->get('filesystem', [
            'directory' => __DIR__ . '/cache',
            'namespace' => 'test',
        ]);

        $this->assertInstanceOf(Adapter\FilesystemAdapter::class, $adapter);
        $this->assertNotSame($adapter, $this->adapters->get('filesystem'));

        $this->assertDirectoryExists(__DIR__ . '/cache/test');

        rmdir(__DIR__ . '/cache/test');
        rmdir(__DIR__ . '/cache');
    }

    public function testCanCreateNullAdapter()
    {
        $this->assertTrue($this->adapters->has('null'));

        $adapter = $this->adapters->get('null');

        $this->assertInstanceOf(Adapter\NullAdapter::class, $adapter);
        $this->assertNotSame($adapter, $this->adapters->get('null'));
    }

    public function testCanCreatePhpArrayAdapter()
    {
        $this->assertTrue($this->adapters->has('phparray'));

        $adapter = $this->adapters->get('phparray', [
            'file' => __DIR__ . '/file.cache',
            'fallback' => 'proxied',
        ]);

        $this->assertInstanceOf(Adapter\PhpArrayAdapter::class, $adapter);
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     */
    public function testPhpArrayWillThrowExceptionWhenFileNotDefined()
    {
        $this->adapters->get('phparray', []);
    }

    public function testCanCreatePhpFilesAdapter()
    {
        $this->assertTrue($this->adapters->has('phpfiles'));

        $adapter = $this->adapters->get('phpfiles');

        $this->assertInstanceOf(Adapter\PhpFilesAdapter::class, $adapter);
        $this->assertNotSame($adapter, $this->adapters->get('phpfiles'));
    }

    public function testCanCreateProxyAdapter()
    {
        $this->assertTrue($this->adapters->has('proxy'));

        $adapter = $this->adapters->get('proxy', [
            'cache' => 'proxied',
        ]);

        $this->assertInstanceOf(Adapter\ProxyAdapter::class, $adapter);
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     */
    public function testProxyAdapterWillThrowExceptionWhenCacheNotDefined()
    {
        $this->adapters->get('proxy', []);
    }

    public function testCanCreateRedisAdapter()
    {
        $this->assertTrue($this->adapters->has('redis'));

        $adapter = $this->adapters->get('redis', [
            'connection' => 'redis://localhost',
            'connection_params' => [
                'persistent' => 0,
            ],
        ]);

        $this->assertInstanceOf(Adapter\RedisAdapter::class, $adapter);
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     */
    public function testRedisAdapterWillThrowExceptionWhenConnectionEmpty()
    {
        $this->adapters->get('redis', [
            'connection' => '',
        ]);
    }
}