<?php
namespace EpTest\Http;

use Ep\Cache\AbstractCacheFactory;
use Ep\Http\AbstractHttpClientFactory;
use Ep\Http\Exception\InvalidResponseException;
use Ep\Http\Middleware\Cache;
use EpTest\ContainerAwareTrait;
use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use PHPUnit\Framework\TestCase;

/**
 * Class AbstractHttpClientFactoryTest
 *
 * @package EpTest\Http
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class AbstractHttpClientFactoryTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testCanCreateClients()
    {
        $container = $this->createContainer([
            'http_clients' => [
                'http.general' => [],
            ],

            'dependencies' => [
                'factories' => [
                    'http.general' => AbstractHttpClientFactory::class,
                ],
            ],
        ]);

        $this->assertTrue($container->has('http.general'));

        $client = $container->get('http.general');

        $this->assertInstanceOf(ClientInterface::class, $client);
        $this->assertSame($client, $container->get('http.general'));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function httpBinIsSlowTestCanCreateConfiguredClients()
    {
        $container = $this->createContainer([
            'http_clients' => [
                'http.bin' => [
                    'base_uri' => 'http://httpbin.org',
                ],
            ],

            'dependencies' => [
                'factories' => [
                    'http.bin' => AbstractHttpClientFactory::class,
                ],
            ],
        ]);

        /** @var Client $client */
        $client = $container->get('http.bin');

        $response = $client->get('/get');
        $this->assertEquals('application/json', $response->getHeaderLine('Content-Type'));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function httpBinIsSlowTestCanCreateCacheableClients()
    {
        $container = $this->createContainer([
            'http_clients' => [
                'http.cached' => [
                    'base_uri' => 'http://httpbin.org',
                    'cache' => 'cache.http',
                ],
            ],

            'caches' => [
                'cache.http' => [
                    'adapter' => 'array',
                    'options' => [
                        'default_lifetime' => 1,
                    ],
                ],
            ],

            'dependencies' => [
                'factories' => [
                    'cache.http' => AbstractCacheFactory::class,
                    'http.cached' => AbstractHttpClientFactory::class,
                ],
            ],
        ]);

        /** @var Client $client */
        $client = $container->get('http.cached');

        $response = $client->get('/get');

        $this->assertTrue($response->hasHeader(Cache::HEADER_CACHE_INFO));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function httpBinIsSlowTestCanCreateValidatedClients()
    {
        $container = $this->createContainer();
        $factory = new AbstractHttpClientFactory();

        $client = $factory($container, '', [
            'base_uri' => 'http://httpbin.org/',
            'validators' => [[
                'name' => 'http_header_value',
                'options' => [
                    'header' => 'Content-Type',
                    'header_value' => 'text/html',
                ],
            ]],
        ]);

        try {
            $client->request('GET', 'response-headers', [
                'query' => [
                    'Content-Type' => 'application/json',
                ],
            ]);
        } catch (InvalidResponseException $e) {
            $this->addToAssertionCount(1);
        }

        $client->request('GET', 'response-headers', [
            'query' => [
                'Content-Type' => 'text/html',
            ],
        ]);
    }
}