<?php
namespace EpTest\Http\Validator;

use Ep\Http\Exception\InvalidResponseException;
use Ep\Http\Validator\HeaderValue;
use Ep\Http\Validator\ValidatorChain;
use Ep\Http\Validator\ValidatorChainFactory;
use EpTest\ContainerAwareTrait;
use GuzzleHttp\Client;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Zend\Diactoros\Response\HtmlResponse;
use Zend\Diactoros\Response\JsonResponse;

/**
 * Class ResponseValidatorTest
 *
 * @package Ep\Http\Middleware
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class ValidatorChainTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * Vraća klijent iskonfiguriran s datim validatorom odgovora i mogućim mock odgovorima.
     *
     * @param ValidatorChain $validator
     * @param array $mockResponses
     *
     * @return ClientInterface
     */
    protected function createClient(ValidatorChain $validator, array $mockResponses = []): ClientInterface
    {
        $stack = HandlerStack::create(new MockHandler($mockResponses));
        $stack->push($validator);

        return new Client([
            'handler' => $stack,
        ]);
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Vraća validator konfiguriran s datim validatorima.
     *
     * @param array $validators
     *
     * @return ValidatorChain
     */
    protected function createValidator(array $validators = []): ValidatorChain
    {
        $container = $this->createContainer();
        $factory = new ValidatorChainFactory();

        /** @noinspection PhpUnhandledExceptionInspection */
        return $factory($container, '', [
            'validators' => $validators,
        ]);
    }

    /**
     * @throws GuzzleException
     */
    public function testWillValidateResponse()
    {
        $validator = $this->createValidator([[
            'name' => 'http_header_value',
            'options' => [
                'header' => 'Content-Type',
                'header_value' => 'application/json',
            ],
        ]]);

        $client = $this->createClient($validator, [
            new HtmlResponse(''),
            new JsonResponse([]),
        ]);

        try {
            $client->request('GET', '/');
        } catch (InvalidResponseException $e) {
            $this->assertInstanceOf(RequestInterface::class, $e->getRequest());
            $this->assertInstanceOf(HtmlResponse::class, $e->getResponse());
            $this->assertArrayHasKey(HeaderValue::INVALID_HEADER_VALUE, $validator->getMessages());
        }

        $response = $client->request('GET', '/');

        $this->assertInstanceOf(ResponseInterface::class, $response);
    }

    public function testChainIsCallable()
    {
        $validator = $this->createValidator();

        $this->assertTrue($validator(''));
        $this->assertInternalType('callable', $validator(function() {}));
    }
}